@@ -1,6 +1,6 @@
 /*
  * melt.c -- MLT command line utility
- * Copyright (C) 2002-2021 Meltytech, LLC
+ * Copyright (C) 2002-2022 Meltytech, LLC
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <limits.h>
 #include <unistd.h>
 #include <signal.h>
+#include <locale.h>
 
 #include <framework/mlt.h>
 
@@ -57,6 +58,11 @@ static void abnormal_exit_handler(int signum)
 	raise( signum );
 }
 
+static void fire_jack_seek_event(mlt_properties jack, int position)
+{
+	mlt_events_fire(jack, "jack-seek", mlt_event_data_from_int(position));
+}
+
 static void transport_action( mlt_producer producer, char *value )
 {
 	mlt_properties properties = MLT_PRODUCER_PROPERTIES( producer );
@@ -74,14 +80,14 @@ static void transport_action( mlt_producer producer, char *value )
 			case 'q':
 			case 'Q':
 				mlt_properties_set_int( properties, "done", 1 );
-				mlt_events_fire( jack, "jack-stop", NULL );
+				mlt_events_fire( jack, "jack-stop", mlt_event_data_none() );
 				break;
 			case '0':
 				position = 0;
 				mlt_producer_set_speed( producer, 1 );
 				mlt_producer_seek( producer, position );
 				mlt_consumer_purge( consumer );
-				mlt_events_fire( jack, "jack-seek", &position, NULL );
+				fire_jack_seek_event(jack, position);
 				break;
 			case '1':
 				mlt_producer_set_speed( producer, -10 );
@@ -99,14 +105,14 @@ static void transport_action( mlt_producer producer, char *value )
 				mlt_producer_set_speed( producer, 0 );
 				mlt_consumer_purge( consumer );
 				mlt_producer_seek( producer, mlt_consumer_position( consumer ) + 1 );
-				mlt_events_fire( jack, "jack-stop", NULL );
+				mlt_events_fire( jack, "jack-stop", mlt_event_data_none() );
 				break;
 			case '6':
 			case ' ':
 				if ( !jack || mlt_producer_get_speed( producer ) != 0 )
 					mlt_producer_set_speed( producer, 1 );
 				mlt_consumer_purge( consumer );
-				mlt_events_fire( jack, "jack-start", NULL );
+				mlt_events_fire( jack, "jack-start", mlt_event_data_none() );
 				break;
 			case '7':
 				mlt_producer_set_speed( producer, 2 );
@@ -140,7 +146,7 @@ static void transport_action( mlt_producer producer, char *value )
 					position = mlt_multitrack_clip( multitrack, mlt_whence_relative_current, 0 );
 					mlt_producer_seek( producer, position );
 					mlt_consumer_purge( consumer );
-					mlt_events_fire( jack, "jack-seek", &position, NULL );
+					fire_jack_seek_event(jack, position);
 				}
 				break;
 			case 'H':
@@ -149,7 +155,7 @@ static void transport_action( mlt_producer producer, char *value )
 					position -= mlt_producer_get_fps( producer ) * 60;
 					mlt_consumer_purge( consumer );
 					mlt_producer_seek( producer, position );
-					mlt_events_fire( jack, "jack-seek", &position, NULL );
+					fire_jack_seek_event(jack, position);
 				}
 				break;
 			case 'h':
@@ -159,8 +165,8 @@ static void transport_action( mlt_producer producer, char *value )
 					mlt_producer_set_speed( producer, 0 );
 					mlt_consumer_purge( consumer );
 					mlt_producer_seek( producer, position );
-					mlt_events_fire( jack, "jack-stop", NULL );
-					mlt_events_fire( jack, "jack-seek", &position, NULL );
+					mlt_events_fire( jack, "jack-stop", mlt_event_data_none() );
+					fire_jack_seek_event(jack, position);
 				}
 				break;
 			case 'j':
@@ -169,7 +175,7 @@ static void transport_action( mlt_producer producer, char *value )
 					position = mlt_multitrack_clip( multitrack, mlt_whence_relative_current, 1 );
 					mlt_consumer_purge( consumer );
 					mlt_producer_seek( producer, position );
-					mlt_events_fire( jack, "jack-seek", &position, NULL );
+					fire_jack_seek_event(jack, position);
 				}
 				break;
 			case 'k':
@@ -178,7 +184,7 @@ static void transport_action( mlt_producer producer, char *value )
 					position = mlt_multitrack_clip( multitrack, mlt_whence_relative_current, -1 );
 					mlt_consumer_purge( consumer );
 					mlt_producer_seek( producer, position );
-					mlt_events_fire( jack, "jack-seek", &position, NULL );
+					fire_jack_seek_event(jack, position);
 				}
 				break;
 			case 'l':
@@ -189,12 +195,12 @@ static void transport_action( mlt_producer producer, char *value )
 					if ( mlt_producer_get_speed( producer ) != 0 )
 					{
 						mlt_producer_set_speed( producer, 0 );
-						mlt_events_fire( jack, "jack-stop", NULL );
+						mlt_events_fire( jack, "jack-stop", mlt_event_data_none() );
 					}
 					else
 					{
 						mlt_producer_seek( producer, position );
-						mlt_events_fire( jack, "jack-seek", &position, NULL );
+						fire_jack_seek_event(jack, position);
 					}
 				}
 				break;
@@ -204,7 +210,7 @@ static void transport_action( mlt_producer producer, char *value )
 					position += mlt_producer_get_fps( producer ) * 60;
 					mlt_consumer_purge( consumer );
 					mlt_producer_seek( producer, position );
-					mlt_events_fire( jack, "jack-seek", &position, NULL );
+					fire_jack_seek_event(jack, position);
 				}
 				break;
 		}
@@ -215,7 +221,7 @@ static void transport_action( mlt_producer producer, char *value )
 	mlt_properties_set_int( properties, "stats_off", 0 );
 }
 
-static void on_jack_started( mlt_properties owner, mlt_consumer consumer, mlt_position *position )
+static void on_jack_started( mlt_properties owner, mlt_consumer consumer, mlt_event_data event_data )
 {
 	mlt_producer producer = mlt_properties_get_data( MLT_CONSUMER_PROPERTIES(consumer), "transport_producer", NULL );
 	if ( producer )
@@ -223,26 +229,28 @@ static void on_jack_started( mlt_properties owner, mlt_consumer consumer, mlt_po
 		if ( mlt_producer_get_speed( producer ) != 0 )
 		{
 			mlt_properties jack = mlt_properties_get_data( MLT_CONSUMER_PROPERTIES( consumer ), "jack_filter", NULL );
-			mlt_events_fire( jack, "jack-stop", NULL );
+			mlt_events_fire( jack, "jack-stop", mlt_event_data_none() );
 		}
 		else
 		{
+			mlt_position position = mlt_event_data_to_int(event_data);
 			mlt_producer_set_speed( producer, 1 );
 			mlt_consumer_purge( consumer );
-			mlt_producer_seek( producer, *position );
+			mlt_producer_seek( producer, position );
 			mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( consumer ), "refresh", 1 );
 		}
 	}
 }
 
-static void on_jack_stopped( mlt_properties owner, mlt_consumer consumer, mlt_position *position )
+static void on_jack_stopped( mlt_properties owner, mlt_consumer consumer, mlt_event_data event_data )
 {
 	mlt_producer producer = mlt_properties_get_data( MLT_CONSUMER_PROPERTIES(consumer), "transport_producer", NULL );
 	if ( producer )
 	{
+		mlt_position position = mlt_event_data_to_int(event_data);
 		mlt_producer_set_speed( producer, 0 );
 		mlt_consumer_purge( consumer );
-		mlt_producer_seek( producer, *position );
+		mlt_producer_seek( producer, position );
 		mlt_properties_set_int( MLT_CONSUMER_PROPERTIES( consumer ), "refresh", 1 );
 	}
 }
@@ -495,6 +503,7 @@ static void show_usage( char *program_name )
 "  -attach-clip filter[:arg] [name=value]*  Attach a filter to a producer\n"
 "  -audio-track | -hide-video               Add an audio-only track\n"
 "  -blank frames                            Add blank silence to a track\n"
+"  -chain id[:arg] [name=value]*            Add a producer as a chain\n"
 "  -consumer id[:arg] [name=value]*         Set the consumer (sink)\n"
 "  -debug                                   Set the logging level to debug\n"
 "  -filter filter[:arg] [name=value]*       Add a filter to the current track\n"
@@ -503,6 +512,7 @@ static void show_usage( char *program_name )
 "  -help                                    Show this message\n"
 "  -jack                                    Enable JACK transport synchronization\n"
 "  -join clips                              Join multiple clips into one cut\n"
+"  -link id[:arg] [name=value]*             Add a link to a chain\n"
 "  -mix length                              Add a mix between the last two cuts\n"
 "  -mixer transition                        Add a transition to the mix\n"
 "  -null-track | -hide-track                Add a hidden track\n"
@@ -523,6 +533,7 @@ static void show_usage( char *program_name )
 "  -repeat times                            Repeat the last cut\n"
 "  -repository path                         Set the directory of MLT modules\n"
 "  -serialise [filename]                    Write the commands to a text file\n"
+"  -setlocale                               Make numeric strings locale-sensitive\n"
 "  -silent                                  Do not display position/transport\n"
 "  -split relative-frame                    Split the last cut into two cuts\n"
 "  -swap                                    Rearrange the last two cuts\n"
@@ -582,19 +593,19 @@ static void query_services( mlt_repository repo, mlt_service_type type )
 	const char *typestr = NULL;
 	switch ( type )
 	{
-		case consumer_type:
+		case mlt_service_consumer_type:
 			services = mlt_repository_consumers( repo );
 			typestr = "consumers";
 			break;
-		case filter_type:
+		case mlt_service_filter_type:
 			services = mlt_repository_filters( repo );
 			typestr = "filters";
 			break;
-		case producer_type:
+		case mlt_service_producer_type:
 			services = mlt_repository_producers( repo );
 			typestr = "producers";
 			break;
-		case transition_type:
+		case mlt_service_transition_type:
 			services = mlt_repository_transitions( repo );
 			typestr = "transitions";
 			break;
@@ -742,6 +753,21 @@ static void set_preview_scale(mlt_profile *profile, mlt_profile *backup_profile,
 	}
 }
 
+static mlt_repository setup_factory(const char* repo_path, int set_locale)
+{
+	mlt_repository repo = mlt_factory_init(repo_path);
+	if (repo && set_locale) {
+		// Load the system locales
+		const char* locale = "";
+#if defined(_WIN32)
+		if (getenv("LC_ALL"))
+			locale = getenv("LC_ALL");
+#endif
+		setlocale( LC_ALL, locale );
+	}
+	return repo;
+}
+
 int main( int argc, char **argv )
 {
 	int i;
@@ -758,12 +784,20 @@ int main( int argc, char **argv )
 	mlt_repository repo = NULL;
 	const char* repo_path = NULL;
 	int is_consumer_explicit = 0;
+	int is_setlocale = 0;
 
 	// Handle abnormal exit situations.
 	signal( SIGSEGV, abnormal_exit_handler );
 	signal( SIGILL, abnormal_exit_handler );
 	signal( SIGABRT, abnormal_exit_handler );
 
+	for (i = 1; i < argc; i++) {
+		if (!strcmp(argv[i], "-setlocale")) {
+			is_setlocale = 1;
+			break;
+		}
+	}
+
 	for ( i = 1; i < argc; i ++ )
 	{
 		// Check for serialisation switch
@@ -784,7 +818,7 @@ int main( int argc, char **argv )
 		{
 			// Construct the factory
 			if ( !repo )
-				repo = mlt_factory_init( repo_path );
+				repo = setup_factory(repo_path, is_setlocale);
 
 			const char *pname = argv[ ++ i ];
 			if ( pname && pname[0] != '-' )
@@ -803,19 +837,19 @@ int main( int argc, char **argv )
 		{
 			// Construct the factory
 			if ( !repo )
-				repo = mlt_factory_init( repo_path );
-		
+				repo = setup_factory(repo_path, is_setlocale);
+
 			const char *pname = argv[ ++ i ];
 			if ( pname && pname[0] != '-' )
 			{
 				if ( !strcmp( pname, "consumers" ) || !strcmp( pname, "consumer" ) )
-					query_services( repo, consumer_type );
+					query_services( repo, mlt_service_consumer_type );
 				else if ( !strcmp( pname, "filters" ) || !strcmp( pname, "filter" ) )
-					query_services( repo, filter_type );
+					query_services( repo, mlt_service_filter_type );
 				else if ( !strcmp( pname, "producers" ) || !strcmp( pname, "producer" ) )
-					query_services( repo, producer_type );
+					query_services( repo, mlt_service_producer_type );
 				else if ( !strcmp( pname, "transitions" ) || !strcmp( pname, "transition" ) )
-					query_services( repo, transition_type );
+					query_services( repo, mlt_service_transition_type );
 				else if ( !strcmp( pname, "profiles" ) || !strcmp( pname, "profile" ) )
 					query_profiles();
 				else if ( !strcmp( pname, "presets" ) || !strcmp( pname, "preset" ) )
@@ -828,13 +862,13 @@ int main( int argc, char **argv )
 					query_vcodecs();
 
 				else if ( !strncmp( pname, "consumer=", 9 ) )
-					query_metadata( repo, consumer_type, "consumer", strchr( pname, '=' ) + 1 );
+					query_metadata( repo, mlt_service_consumer_type, "consumer", strchr( pname, '=' ) + 1 );
 				else if ( !strncmp( pname, "filter=", 7 ) )
-					query_metadata( repo, filter_type, "filter", strchr( pname, '=' ) + 1 );
+					query_metadata( repo, mlt_service_filter_type, "filter", strchr( pname, '=' ) + 1 );
 				else if ( !strncmp( pname, "producer=", 9 ) )
-					query_metadata( repo, producer_type, "producer", strchr( pname, '=' ) + 1 );
+					query_metadata( repo, mlt_service_producer_type, "producer", strchr( pname, '=' ) + 1 );
 				else if ( !strncmp( pname, "transition=", 11 ) )
-					query_metadata( repo, transition_type, "transition", strchr( pname, '=' ) + 1 );
+					query_metadata( repo, mlt_service_transition_type, "transition", strchr( pname, '=' ) + 1 );
 				else if ( !strncmp( pname, "profile=", 8 ) )
 					query_profile( strchr( pname, '=' ) + 1 );
 				else if ( !strncmp( pname, "preset=", 7 ) )
@@ -845,10 +879,10 @@ int main( int argc, char **argv )
 			else
 			{
 query_all:
-				query_services( repo, consumer_type );
-				query_services( repo, filter_type );
-				query_services( repo, producer_type );
-				query_services( repo, transition_type );
+				query_services( repo, mlt_service_consumer_type );
+				query_services( repo, mlt_service_filter_type );
+				query_services( repo, mlt_service_producer_type );
+				query_services( repo, mlt_service_transition_type );
 				fprintf( stdout, "# You can query the metadata for a specific service using:\n"
 					"# -query <type>=<identifier>\n"
 					"# where <type> is one of: consumer, filter, producer, or transition.\n" );
@@ -875,7 +909,7 @@ query_all:
 		else if ( !strcmp( argv[ i ], "-version" ) || !strcmp( argv[ i ], "--version" ) )
 		{
 			fprintf( stdout, "%s " VERSION "\n"
-				"Copyright (C) 2002-2021 Meltytech, LLC\n"
+				"Copyright (C) 2002-2022 Meltytech, LLC\n"
 				"<https://www.mltframework.org/>\n"
 				"This is free software; see the source for copying conditions.  There is NO\n"
 				"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n",
@@ -909,7 +943,7 @@ query_all:
 
 	// Construct the factory
 	if ( !repo )
-		repo = mlt_factory_init( repo_path );
+		repo = setup_factory(repo_path, is_setlocale);
 
 	// Create profile if not set explicitly
 	if ( getenv( "MLT_PROFILE" ) )
@@ -1082,7 +1116,7 @@ query_all:
 		error = mlt_properties_get_int( MLT_CONSUMER_PROPERTIES( consumer ), "melt_error" );
 		mlt_consumer_connect( consumer, NULL );
 		if ( !is_abort )
-			mlt_events_fire( MLT_CONSUMER_PROPERTIES(consumer), "consumer-cleanup", NULL);
+			mlt_events_fire( MLT_CONSUMER_PROPERTIES(consumer), "consumer-cleanup", mlt_event_data_none());
 	}
 
 	if ( is_abort )
