@@ -31,10 +31,10 @@
  *
  * image scaler implementations are expected to support the following in and out formats:
  * yuv422 -> yuv422
- * rgb24 -> rgb24
- * rgb24a -> rgb24a
- * rgb24 -> yuv422
- * rgb24a -> yuv422
+ * rgb -> rgb
+ * rgba -> rgba
+ * rgb -> yuv422
+ * rgba -> yuv422
  */
 
 typedef int ( *image_scaler )( mlt_frame frame, uint8_t **image, mlt_image_format *format, int iwidth, int iheight, int owidth, int oheight );
@@ -172,7 +172,7 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 		int iheight = *height;
 		int owidth = *width;
 		int oheight = *height;
-		char *interps = mlt_properties_get( properties, "rescale.interp" );
+		char *interps = mlt_properties_get( properties, "consumer.rescale" );
 
 		if ( mlt_properties_get( filter_properties, "factor" ) )
 		{
@@ -185,7 +185,7 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 		if ( interps == NULL )
 		{
 			interps = mlt_properties_get( MLT_FILTER_PROPERTIES( filter ), "interpolation" );
-			mlt_properties_set( properties, "rescale.interp", interps );
+			mlt_properties_set( properties, "consumer.rescale", interps );
 		}
 	
 		// If meta.media.width/height exist, we want that as minimum information
@@ -211,7 +211,7 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 		// Deinterlace if height is changing to prevent fields mixing on interpolation
 		// One exception: non-interpolated, integral scaling
 		if ( iheight != oheight && ( strcmp( interps, "nearest" ) || ( iheight % oheight != 0 ) ) )
-			mlt_properties_set_int( properties, "consumer_deinterlace", 1 );
+			mlt_properties_set_int( properties, "consumer.progressive", 1 );
 
 		// Convert the image to yuv422 when using the local scaler
 		if ( scaler_method == filter_scale )
@@ -221,7 +221,7 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 		mlt_frame_get_image( frame, image, format, &iwidth, &iheight, writable );
 
 		// Get rescale interpretation again, in case the producer wishes to override scaling
-		interps = mlt_properties_get( properties, "rescale.interp" );
+		interps = mlt_properties_get( properties, "consumer.rescale" );
 	
 		if ( *image && strcmp( interps, "none" ) && ( iwidth != owidth || iheight != oheight ) )
 		{
