@@ -1,6 +1,6 @@
 /*
  * filter_resize.c -- resizing filter
- * Copyright (C) 2003-2020 Meltytech, LLC
+ * Copyright (C) 2003-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -85,7 +85,7 @@ static void resize_image( uint8_t *output, int owidth, int oheight, uint8_t *inp
 		return;
 	}
 
-	if ( format == mlt_image_rgb24a )
+	if ( format == mlt_image_rgba )
 	{
 		memset(p, 0, size * bpp);
 		if (alpha_value != 0) {
@@ -160,7 +160,7 @@ static uint8_t *frame_resize_image( mlt_frame frame, int owidth, int oheight, ml
 		mlt_frame_set_image( frame, output, owidth * ( oheight + 1 ) * bpp, mlt_pool_release );
 
 		// We should resize the alpha too
-		if ( format != mlt_image_rgb24a && alpha && alpha_size >= iwidth * iheight )
+		if ( format != mlt_image_rgba && alpha && alpha_size >= iwidth * iheight )
 		{
 			alpha = resize_alpha( alpha, owidth, oheight, iwidth, iheight, alpha_value );
 			if ( alpha )
@@ -210,15 +210,8 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 	// Reset the aspect ratio
 	mlt_properties_set_double( properties, "aspect_ratio", aspect_ratio );
 
-	// XXX: This is a hack, but it forces the force_full_luma to apply by doing a RGB
-	// conversion because range scaling only occurs on YUV->RGB. And we do it here,
-	// after the deinterlace filter, which only operates in YUV to avoid a YUV->RGB->YUV->?.
-	// Instead, it will go YUV->RGB->?.
-	if ( mlt_properties_get_int( properties, "force_full_luma" ) )
-		*format = mlt_image_rgb24a;
-
-	// Hmmm...
-	char *rescale = mlt_properties_get( properties, "rescale.interp" );
+	// Skip scaling if requested
+	char *rescale = mlt_properties_get( properties, "consumer.rescale" );
 	if ( rescale != NULL && !strcmp( rescale, "none" ) )
 		return mlt_frame_get_image( frame, image, format, width, height, writable );
 
@@ -284,6 +277,11 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 	{
 		*image = frame_resize_image( frame, *width, *height, *format );
 	}
+	else
+	{
+		*width = owidth;
+		*height = oheight;
+	}
 
 	return error;
 }
