@@ -1,6 +1,7 @@
 /*
  * producer_frei0r.c -- frei0r producer
  * Copyright (c) 2009 Jean-Baptiste Mardelle <jb@kdenlive.org>
+ * Copyright (C) 2009-2019 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,7 +27,7 @@
 
 static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_format *format, int *width, int *height, int writable )
 {
-    mlt_producer producer = mlt_frame_pop_service( frame );
+	mlt_producer producer = mlt_frame_pop_service( frame );
 
 	// Choose suitable out values if nothing specific requested
 	if ( *width <= 0 )
@@ -35,7 +36,7 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 		*height = mlt_service_profile( MLT_PRODUCER_SERVICE(producer) )->height;
 
 	// Allocate the image
-	*format = mlt_image_rgb24a;
+	*format = mlt_image_rgba;
 	int size = mlt_image_format_size( *format, *width, *height, NULL );
 
 	// Allocate the image
@@ -46,14 +47,14 @@ static int producer_get_image( mlt_frame frame, uint8_t **buffer, mlt_image_form
 
 	if ( *buffer != NULL )
 	{
-		double position = mlt_frame_get_position( frame );
+		mlt_position position = mlt_frame_get_position( frame );
 		mlt_profile profile = mlt_service_profile( MLT_PRODUCER_SERVICE( producer ) );
-		double time = position / mlt_profile_fps( profile );
+		double time = (double) position / mlt_profile_fps( profile );
 		int length = mlt_producer_get_playtime( producer );
 		process_frei0r_item( MLT_PRODUCER_SERVICE(producer), position, time, length, frame, buffer, width, height );
 	}
 
-    return 0;
+	return 0;
 }
 
 int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int index )
@@ -66,9 +67,6 @@ int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int index )
 		// Obtain properties of frame and producer
 		mlt_properties properties = MLT_FRAME_PROPERTIES( *frame );
 
-		// Set the producer on the frame properties
-		mlt_properties_set_data( properties, "producer_frei0r", producer, 0, NULL, NULL );
-
 		// Update timecode on the frame we're creating
 		mlt_frame_set_position( *frame, mlt_producer_position( producer ) );
 
@@ -76,9 +74,11 @@ int producer_get_frame( mlt_producer producer, mlt_frame_ptr frame, int index )
 		mlt_properties_set_int( properties, "progressive", 1 );
 		mlt_profile profile = mlt_service_profile( MLT_PRODUCER_SERVICE( producer ) );
 		mlt_properties_set_double( properties, "aspect_ratio", mlt_profile_sar( profile ) );
+		mlt_properties_set_int( properties, "meta.media.width", profile->width );
+		mlt_properties_set_int( properties, "meta.media.height", profile->height );
 
 		// Push the get_image method
-        mlt_frame_push_service( *frame, producer );
+		mlt_frame_push_service( *frame, producer );
 		mlt_frame_push_get_image( *frame, producer_get_image );
 	}
 
