@@ -1,6 +1,7 @@
 /*
  * filter_tcolor.c -- tcolor filter
  * Copyright (c) 2007 Marco Gittler <g.marco@freenet.de>
+ * Copyright (c) 2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,11 +20,38 @@
 
 #include <framework/mlt_filter.h>
 #include <framework/mlt_frame.h>
+#include <framework/mlt_slices.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 
+typedef struct {
+	uint8_t* image;
+	int width;
+	int height;
+	double over_cr;
+	double over_cb;
+} slice_desc;
+
+static int do_slice_proc(int id, int index, int jobs, void* data)
+{
+	(void) id; // unused
+	slice_desc* desc = (slice_desc*) data;
+	int slice_line_start, slice_height = mlt_slices_size_slice(jobs, index, desc->height, &slice_line_start);
+	int slice_line_end = slice_line_start + slice_height;
+	int line_size = desc->width * 2;
+	int x,y;
+	for ( y = slice_line_start; y < slice_line_end; y++)
+	{
+		uint8_t* p = desc->image + y * line_size;
+		for ( x = 0; x < line_size; x += 4)
+		{
+			p[x+1] = CLAMP( ((double)p[x+1] - 127.0) * desc->over_cb + 127.0, 0, 255);
+			p[x+3] = CLAMP( ((double)p[x+3] - 127.0) * desc->over_cr + 127.0, 0, 255);
+		}
+	}
+}
 
 static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format *format, int *width, int *height, int writable )
 {
@@ -37,24 +65,13 @@ static int filter_get_image( mlt_frame frame, uint8_t **image, mlt_image_format
 
 	if ( error == 0 && *image )
 	{
-		double over_cr = mlt_properties_anim_get_double( properties, "oversaturate_cr", pos, len )/100.0;
-		double over_cb = mlt_properties_anim_get_double( properties, "oversaturate_cb", pos, len )/100.0;
-
-		int video_width = *width;
-		int video_height = *height;
-
-		int x,y;
-		
-		for ( y = 0; y < video_height; y++)
-		{
-			for ( x = 0; x < video_width; x += 2)
-			{
-				uint8_t *pix = (*image + y * video_width * 2 + x * 2 + 1);
-				uint8_t *pix1 = (*image + y * video_width * 2 + x * 2 + 3);
-				*pix = MIN(MAX( ((double) * pix - 127.0) * over_cb + 127.0,0), 255);
-				*pix1 = MIN(MAX( ((double) * pix1 - 127.0) * over_cr + 127.0,0), 255);
-			}
-		}
+		slice_desc desc;
+		desc.over_cr = mlt_properties_anim_get_double( properties, "oversaturate_cr", pos, len )/100.0;
+		desc.over_cb = mlt_properties_anim_get_double( properties, "oversaturate_cb", pos, len )/100.0;
+		desc.image = *image;
+		desc.width = *width;
+		desc.height = *height;
+		mlt_slices_run_normal(0, do_slice_proc, &desc);
 	}
 
 	return error;
