@@ -1,6 +1,5 @@
 /*
- * Copyright (c) 2015-2020 Meltytech, LLC
- * Author: Brian Matherly <code@brianmatherly.com>
+ * Copyright (c) 2015-2022 Meltytech, LLC
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,13 +20,17 @@
 #define GRAPH_H
 
 #include <framework/mlt.h>
+#include <QColor>
 #include <QPainter>
 #include <QRectF>
+#include <QVector>
 
+QVector<QColor> get_graph_colors( mlt_properties filter_properties );
 void setup_graph_painter( QPainter& p, QRectF& rect, mlt_properties filter_properties );
 void setup_graph_pen( QPainter& p, QRectF& rect, mlt_properties filter_properties, double scale );
 void paint_line_graph( QPainter& p, QRectF& rect, int points, float* values, double tension, int fill );
 void paint_bar_graph( QPainter& p, QRectF& rect, int points, float* values );
+void paint_segment_graph( QPainter& p, const QRectF& rect, int points, const float* values, const QVector<QColor>& colors, int segments, int segment_gap, int segment_width );
 
 
 #endif // GRAPH_H
