@@ -159,7 +159,6 @@ public:
 		m_pen.setWidthF(outline);
 		m_font = font;
 		m_lineSpacing = lineSpacing + m_metrics.lineSpacing();
-		m_path.setFillRule(Qt::WindingFill);
 		m_align = align;
 		m_width = width;
 		updateText(text);
@@ -189,6 +188,7 @@ public:
 			}
 			m_path.addPath(linePath);
 		}
+		m_path.setFillRule(Qt::WindingFill);
 	}
 
 	virtual QRectF boundingRect() const
@@ -207,7 +207,7 @@ public:
 		painter->fillPath(m_path, m_brush);
 		if ( m_outline > 0 )
 		{
-			painter->strokePath(m_path, m_pen);
+			painter->strokePath(m_path.simplified(), m_pen);
 		}
 	}
 
